#include "acha.h"
#include "cm.h"
#include "sm.h"
#include "tm.h"
#include "fm.h"

/*
 * 메인함수 : ACHA 프로그램의 메인
 */
#ifndef DEBUG
int main(int argc, char **argv)
{
	CommunicationManager cm;
	cm.run();

	return 0;
}
#endif

/*
 * 함수 : time_t 형 시간을 읽기 좋게 변경
 * 인자 : time_t 형 시간형식
 * 반환 : yymmdd-hh:mm:ss 형식의 문자열
 */
char *timeFormat( time_t now )
{
	struct tm *timeTmp = localtime( &now );
	static char buf[SDATA];
	bzero( buf, SDATA );

	sprintf( buf,
			 "%02d%02d%02d-%02d:%02d:%02d",
			 timeTmp->tm_year-100,
			 timeTmp->tm_mon+1,
			 timeTmp->tm_mday,
			 timeTmp->tm_hour,
			 timeTmp->tm_min,
			 timeTmp->tm_sec );

	return buf;
}

/*
 * 함수 : 타겟의 IP address를 받아 온다.
 * 설명 : 유효성도 체크한다.
 * 인자 : IP 혹은 Domain형식의 string
 * 반환 : IP 형식의 char형 포인터
 *        NULL : 유효하지 않은 입력
 */
char *getTargetIP( char *target )
{
	if( isIPformat( target ) )
	{
		return getTargetIP( target );
	}
	else
	{
		return NULL;
	}
}

/*
 * 함수 : 도메인 네임을 IP로 변환
 * 인자 : 도메인 네임형 스트링
 * 반환 : IP 형식의 char형 포인터
 */
char *getIPfromDomain( char *domain )
{
	struct hostent *he;
	static char buf[SDATA];

	bzero( buf, SDATA );
	he = gethostbyname( domain );
	strncpy( buf, inet_ntoa( *(struct in_addr*)he->h_addr), SDATA );

	return buf;
}

/*
 * 함수 : 입력받은 스트링이 IP형식인지 확인
 * 반환 : IP형식 - true    else   false
 */
bool isIPformat( char *str )
{
	int a, b, c, d;
	sscanf( str, "%d.%d.%d.%d", &a, &b, &c, &d );
	if( a >  0  && a < 256  &&
		b >= 0  && b < 256  &&
		c >= 0  && d < 256  &&
		d >  0  && d < 256 )
	{
		return true;
	}

	return false;
}

/*
 * 함수 : 인자 갯수를 리턴한다.
 */
char *gen_rpath( char *lp, char *rp, char *fullpath )
{
	static char buf[DATASIZ];
	bzero( buf, DATASIZ );

	char *filepath = &fullpath[strlen(lp)];
	if( rp[strlen(rp)-1] == '/' && filepath[0] == '/' )
	{
		sprintf( buf, "%s%s", rp, &filepath[1] );

	}
	else if( ( rp[strlen(rp)-1] != '/' && filepath[0] == '/' ) || 
			 ( rp[strlen(rp)-1] == '/' && filepath[0] != '/' ) )
	{
		sprintf( buf, "%s%s", rp, filepath );
	}
	else if( rp[strlen(rp)-1] != '/' && filepath[0] != '/' )
	{
		sprintf( buf, "%s/%s", rp, filepath );
	}

	return buf;
}

/*
 * 함수 : 인자 갯수를 리턴한다.
 */
int argc( char *str )
{
	int count;
	char *tok;
	char buf[BUFSIZ] = "";
	strncpy( buf, str, BUFSIZ );

	tok = strtok( buf, DELIM );
	for( count = 0; tok != NULL; count++ )
	{
		tok = strtok( NULL, DELIM );
	}

	return count;
}

/*
 * 함수 : 텍스트 출력 컬러 설정
 */
void setcolor( int color )
{
	cout << "[00m";

	switch( color )
	{
		case COLOR_RED :
			cout << "[1;31m";
			break;
		case COLOR_GREEN:
			cout << "[1;32m";
			break;
		case COLOR_YELLOW :
			cout << "[1;33m";
			break;
		case COLOR_BLUE :
			cout << "[1;34m";
			break;
		case COLOR_PINK :
			cout << "[1;35m";
			break;
		case COLOR_SKY :
			cout << "[1;36m";
			break;
		case COLOR_WHITE :
			cout << "[1;37m";
			break;
		case COLOR_DARKRED :
			cout << "[31m";
			break;
		case COLOR_DARKGREEN :
			cout << "[32m";
			break;
		case COLOR_BROWN :
			cout << "[33m";
			break;
		case COLOR_DARKBLUE :
			cout << "[34m";
			break;
		case COLOR_PUPLE :
			cout << "[35m";
			break;
		case COLOR_DARKSKY :
			cout << "[36m";
			break;
		case COLOR_GRAY :
			cout << "[37m";
			break;
		default :
			cout << "[00m";
			break;
	}

	return;
}

/*
 * 함수 : string 의 MD5 해쉬코드를 리턴
 */
char *md5str( char *str )
{
	unsigned char *originStr = (unsigned char *)strdup(str);
	unsigned char md5[16];
	unsigned long len;
    static char md5msg[40];
	int i;

	memset(md5, 0, 16);

	len = strlen(str);
	MD5(originStr, len, md5);

	bzero( md5msg, 40 );
	for( i = 0; i < 16; i ++ )
	{
        sprintf(md5msg+(i*2), "%02X", md5[i]);
	}

	return md5msg;
}

/*
 * 함수 : file 의 MD5 해쉬코드를 리턴
 */
char *md5file( char *path )
{
    MD5_CTX lctx;
    unsigned char digest[16];
    static char md5msg[40];
    unsigned int i;
    int fd;
    struct stat status;
    char *data;

    MD5_Init(&lctx);

    fd = open(path, O_RDONLY);
    if (fd < 0)
    {
		cout << "MD5 :: Can't Open File" << endl;
		return NULL;
    }
    fstat(fd, &status);

    data = (char *)malloc(status.st_size);
    read (fd, data, status.st_size);

    MD5_Update(&lctx, data, status.st_size);
    MD5_Final(digest, &lctx);

	bzero( md5msg, 40 );
    for (i = 0; i < sizeof(digest); ++i)
    {
        sprintf(md5msg+(i*2), "%02X", digest[i]);
    }

    free( data );
	close( fd );

    return md5msg;
}

