#ifndef __ACHA_H__
#define __ACHA_H__

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include <arpa/inet.h>
#include <signal.h>
#include <fcntl.h>
#include <wait.h>
#include <pthread.h>
#include <errno.h>
#include <fcntl.h>
#include <termios.h>
#include <openssl/md5.h>

#include <iostream>
#include <string>
#include <deque>
#include <list>
#include <vector>
#include <algorithm>

using namespace std;

#define CONTROL_PORT		8887
#define CONTROL_PORT_END	8787
#define CM_PORT				8888
#define CM_PORT_END			8988
#define CM_RLOCK_PATH		"/tmp/ACHA"
#define CM_PASS_LEN			0					// 패스워드의 최소길이
//#define CM_PASS_LEN			6
#define SDATA				256					// 작은데이터사이즈
#define DATASIZ				1024				// 패킷용 데이터사이즈
#define REMOTE				"127.0.0.1"			// 기본 원격지 주소
#define CONFIGDIR			"config"			// 설정파일 경로
#define	SYNCLISTPATH		"config/SyncList"	// Synclist 경로
#define	CFLISTPATH			"config/ChangedFiles"
#define	DELIM				" |\t\n"
//#define MAXPATHLEN 			50
#define	MAX_FILENAME			256
#define TEMPFOLDER 			"temp"
#define TMTEMP	 			"temp/tm_temp_file"

enum {
	CM_ACKNO,			// 명령 실행 불가 응답
	CM_ACKOK, 			// 명령 실행 완료 응답
	CM_UNKNOWN,			// 처리 할 수 없는 명령 응담
	CM_LOCK,			// 락 설정 요청
	CM_UNLOCK,			// 락 해제 요청
	CM_ADDSYNC,			// 동기화 추가 요청
	CM_DELSYNC,			// 동기화 삭제 요청
	CM_ADDFILE,			// 파일 추가 요청
	CM_ADDDIR,			// 디렉토리 추가 요청
	CM_DELFILE,			// 파일 삭제 요청
	CM_DELDIR,			// 디렉토리 삭제 요청
	CM_SENDDATA,		// 파일 전송
	CM_MD5HASH,			// 해쉬코드 같음. 전송 필요 없음
	CM_UPDATESYNCTIME,	// 동기화 시간 갱신
	CM_FTPREADY,		// FTP 서버 실행 요청

	CM_ERROR,			// 함수의 비정상 종료

	TM_INFO = 1000,		// 전송파일의 정보
	TM_OK	= 1001,		// 파일 수신 완료
	TM_NO	= 1002,		// 파일 수신 실패

	CM_TEST,

	LEN_CONTINUE = 10000// 계속되는 파일 전송
};

enum {
	FLAG_SYNC,			// SyncFlag의 일반적인 상황
	FLAG_NEWSYNC,		// local에는 생성, remote에는 아직 미생성
	FLAG_DELSYNC,		// remote의 응답이 없어서 제거 대기상태
};

enum { 
	DIR_COPY = 1,
	DIR_FILE = 2,
	DIR_DIR = 4, 
	DIR_DELETE = 8
};

enum { 
	CI_DIRADD  = 0,
	CI_FILEADD = 1,
	CI_FILEDEL = 2, 
	CI_DIRDEL  = 3
};

enum {
	COLOR_NONE,
	COLOR_RED,
	COLOR_GREEN,
	COLOR_YELLOW,
	COLOR_BLUE,
	COLOR_PINK,
	COLOR_SKY,
	COLOR_WHITE,
	COLOR_DARKRED,
	COLOR_DARKGREEN,
	COLOR_BROWN,
	COLOR_DARKBLUE,
	COLOR_PUPLE,
	COLOR_DARKSKY,
	COLOR_GRAY
};

enum {
	SHOW_SCAN,
	SHOW_LAST,
	SHOW_CHANGE
};

enum {
	FLAG_ADD = 1,
	FLAG_DELETE = 2,
	FLAG_CHANGE_OPTION = 4,
	FLAG_CHANGE_FILESIZE = 8,
	FLAG_ON_ADD,
	FLAG_OFF_ADD,
	FLAG_ON_DELETE,
	FLAG_OFF_DELETE,
	FLAG_ON_CHANGE_OPTION,
	FLAG_OFF_CHANGE_OPTION,
	FLAG_ON_CHANGE_FILESIZE,
	FLAG_OFF_CHANGE_FILESIZE,
};

//////////////////////////////////////////////////////
// Communication Manager
//
// 전송단위 메시지 구조체
typedef struct _cmdData {
	unsigned int type;
	unsigned int len;
	char data[DATASIZ];
} cmdData;

// 접속중인 클라이언트 정보
typedef struct _clientInfo {
	pthread_t tid;
	int clientSockFd;
	string clientIP;
//	int serverSockFd;
} clientInfo;
//////////////////////////////////////////////////////

typedef struct _syncListnode {
	char *syncName;			// Sync Name ( identifier )
	char *target;			// IP or Domain
	char *localPath;		// Sync Local Directory
	char *remotePath;		// Sync Remote Directory
	char *userId;			// User Id
	char *password;			// password
	int fileSize;			// file size(byte)
	int option;				// sync option (최신파일우선, 파일 크기우선 등 겹쳤을 때 처리되는 방식)
	int Flag;				// Sync Option
	time_t lastSyncTime;	// Last Syncronized Time
} syncListNode;

// 로컬 파일 정보 구조체 : 차후 list로 관리
typedef struct _scanfileinfo
{
	int type;					// 디렉토리냐 파일이냐 등등
	char name[MAX_FILENAME];	// 파일명
	time_t mod_time;			// 수정된 시간
} scanInfo;

// FM과 CM간의 통신에 사용되는 구조체로 ChangedList를 생성한다.
typedef struct _changedinfo
{
	char lPath[MAX_FILENAME];
	char rPath[MAX_FILENAME];
	int type;
} changeInfo;		

char *md5file( char * );		// 파일 MD5 해쉬코드
char *md5str( char * );			// 스트링 MD5 해쉬코드
char *gen_rpath( char *, char *, char * );// 원격지 path 만들기
char *timeFormat( time_t );		// return readable format
char *getTargetIP( char* );		// 도메인 네임으로 IP를 받음
char *getIPfromDomain( char * );// domain -> IP address
bool isIPformat( char * );		// 입력값이 IP형식인지 체크
void setcolor( int );			// 칼라 적용
int argc( char * );				// 인자의 개수를 리턴

#endif

