/*===================================//
 *
 * cm.cpp
 *
 * commnunication 을 위한 프로그램
 *
 * 네트워크 관련 모든 통신을 책임진다.
 *
 * Written by JH. Cho.
 *
 *===================================*/

#include "acha.h"
#include "cm.h"
#include "tm.h"

#ifdef DEBUG
int main( void )
{
	CommunicationManager cm;
	cm.run();

	return 0;
}
#endif

/*
 * 함수 : 실행
 * 설명 : 의미없음. CM의 기본 실행 루틴
 */
void CommunicationManager::run( void )
{
	init();

	init_server();

	bind_server();

//	pthread_create( &timer_tid, NULL, CommunicationManager::timer, this );

	status_cs = pthread_create( &cm_tid, 
								NULL, 
								CommunicationManager::accept_server,
								this );
	status_sm = pthread_create( &sm_tid,
								NULL,
								SyncManager::run,
								this );

	while( getCmd() )
	{
		receiveDataParsing();
	}

	cleanUp();

	return;
}

/*
 * 함수 : 레코드락
 * 설명 : 프로그램 중복실행 방지를 위한 함수
 */
bool CommunicationManager::recordLock( void )
{
	fdRecordLock = open( recordLockPath, O_CREAT|O_TRUNC|O_RDWR, 0600 );

	if( flock( fdRecordLock, LOCK_EX|LOCK_NB ) < 0 )
	{
		cout << "프로그램이 이미 실행중입니다" << endl;
		return false;
	}
	return true;
}

/*
 * 함수 : 레코드락 해제
 * 설명 : 프로그램 중복실행 방지를 위한 함수
 */
bool CommunicationManager::recordUnlock( void )
{
	// record lock 제거
	flock( fdRecordLock, LOCK_UN );

	close( fdRecordLock );
	fdRecordLock = -1;

	unlink( recordLockPath );

	return true;
}

/*
 * 함수 : 사용자로부터 명령을 입력 받음
 * 설명 : UI와 network, 그외 다른 클래스로부터 명령을 입력 받음
 *        디버깅을 위한 함수
 */
bool CommunicationManager::getCmd( void )
{
	if( demon ) pause();

	bzero( cmdBuffer, sizeof(cmdBuffer) );

	setcolor( COLOR_GREEN );
	cout << "CM::getCmd() > ";
	setcolor( COLOR_NONE );

	fgets( cmdBuffer, DATASIZ, stdin );
	cmdBuffer[strlen(cmdBuffer)-1] = '\0';		// cmdBuffer 에서 '\n' 제거

	if( strncmp( cmdBuffer, "exit", strlen("exit") ) == 0 )
	{
		return false;
	}
	
	return true;
}

/*
 * 함수 : 사용자로부터 명령을 입력 받음
 * 설명 : UI와 network, 그외 다른 클래스로부터 명령을 입력 받음
 *        디버깅을 위한 함수
 */
int CommunicationManager::getCmd( char* cmd )
{
	bzero( cmdBuffer, sizeof(cmdBuffer) );
	strncpy( cmdBuffer, cmd, sizeof(cmdBuffer) );
	
	return receiveDataParsing();
}

/*
 * 함수 : 받은 데이터 파싱
 * 설명 : 받은 데이터에 대해 해당 함수를 호출한다.
 *        전송 받은 데이터는 멤버변수에 저장된다.
 * 반환 : SendCmd 형 플래그
 */
int CommunicationManager::receiveDataParsing( void )
{
	int cmdLength = strlen( cmdBuffer );

	// 엔터 무시
	if( cmdLength == 0 )
	{
		return true;
	}

	if( strcmp( cmdBuffer, "demon" ) == 0 )
	{
		demon = true;
		if( fork() != 0 )
			exit(0);
		else
			setsid();
	}

	if( strcmp( cmdBuffer, "debug" ) == 0 ||
		strcmp( cmdBuffer, "d" ) == 0 ) 
	{
		debug();
	}

	if( strcmp( cmdBuffer, "port refresh" ) == 0  ||
		strcmp( cmdBuffer, "pr" ) == 0 )
	{
		portRefresh();
	}

	if( strcmp( cmdBuffer, "lock" ) == 0 )
	{
		if( isLock )
		{
			return CM_ACKNO;
		}
		isLock = true;

		sm->lock();
		return CM_ACKOK;
	}
	else if( strcmp( cmdBuffer, "list" ) == 0 ||
			 strcmp( cmdBuffer, "l" ) 	 == 0 )
	{
		sm->showList();
	}

	if( strcmp( cmdBuffer, "unlock" ) == 0 )
	{
		isLock = false;
		sm->unlock();
		return CM_ACKOK;
	}
	
	if( strcmp( cmdBuffer, "debug on" ) == 0 )
	{
		debugOn();
	}
	
	if( strcmp( cmdBuffer, "debug off" ) == 0 )
	{
		debugOff();
	}

	if( strcmp( cmdBuffer, "record lock" ) == 0 ||
	    strcmp( cmdBuffer, "rl" ) == 0 )
	{
		recordLock();
	}

	if( strcmp( cmdBuffer, "record unlock" ) == 0 ||
		strcmp( cmdBuffer, "rul" ) == 0 )
	{
		recordUnlock();
	}

	if( strcmp( cmdBuffer, "set pass" ) == 0 )
	{
		setPassword();
	}

	if( strcmp( cmdBuffer, "get pass" ) == 0 )
	{
		getPassword();
	}

	if( strcmp( cmdBuffer, "test" ) == 0 )
	{

	}

	if( strncmp( cmdBuffer, "port ", strlen("port ") ) == 0  && 
		argc( cmdBuffer ) == 2 )
	{
		sscanf( &cmdBuffer[5], "%d", &sendPort );
	}

	if( strncmp( cmdBuffer, "target ", strlen("target ") ) == 0  && 
		argc( cmdBuffer ) == 2 )
	{
		bzero( remoteTarget, sizeof(remoteTarget) );
		sscanf( &cmdBuffer[7], "%s", remoteTarget );
		return CM_ACKOK;
	}

	if( strcmp( cmdBuffer, "connect" ) == 0 )
	{
		if( debugMode )
		{
			setcolor( COLOR_YELLOW );
			cout << "CM::connect : " << remoteTarget << endl;
			setcolor( COLOR_NONE );
		}

		if( init_client() )
		{
			return CM_ACKNO;
		}

		if( connect_client() )
		{
			return CM_ACKNO;
		}
		else
		{
			return CM_ACKOK;
		}
	}

	if( strcmp( cmdBuffer, "disconnect" ) == 0 )
	{
		if( debugMode )
		{
			setcolor( COLOR_YELLOW );
			cout << "CM::disconnect" << endl;
			setcolor( COLOR_NONE );
		}
		disconnect_client();
		return CM_ACKOK;
	}

	if( strncmp( cmdBuffer, "send ", strlen("send ") ) == 0  && 
		argc( cmdBuffer ) >= 2 )
	{
		if( strcmp( &cmdBuffer[5], "lock" ) == 0 )
		{
			clientData.type = CM_LOCK;
		}
		else if( strcmp( &cmdBuffer[5], "unlock" ) == 0 )
		{
			clientData.type = CM_UNLOCK;
		}
		else
		{
			clientData.type = CM_TEST;
		}

		clientData.len  = strlen( cmdBuffer ) - strlen("send ");
		strncpy( clientData.data, &cmdBuffer[strlen("send ")], DATASIZ );
		return sendCmd();
	}

	if( strncmp( cmdBuffer, "md5str ", strlen("md5str ") ) == 0  && 
		argc( cmdBuffer ) >= 2 )
	{
		char *str = strdup( &cmdBuffer[7] );
		cout << md5str( str ) << endl;
		free( str );
	}
	
	if( strncmp( cmdBuffer, "md5file ", strlen("md5file ") ) == 0  && 
		argc( cmdBuffer ) == 2 )
	{
		cout << md5file( &cmdBuffer[8] ) << endl;
	}

	// 디렉토리 추가
	if( strncmp( cmdBuffer, "adddir ", strlen("adddir ") ) == 0  && 
		argc( cmdBuffer ) == 2 )
	{
		clientData.type = CM_ADDDIR;
		bzero( clientData.data, sizeof(clientData.data) );

		char filename[DATASIZ] = "";
		struct stat status;
		mode_t filePermission;

		sscanf( &cmdBuffer[7], "%s", filename );
		stat( filename, &status );
		filePermission 	= status.st_mode;
		sprintf( clientData.data, "%s %d", filename, filePermission );
		clientData.len  = strlen( clientData.data );
		return sendCmd();
	}

	// 디렉토리 삭제
	if( strncmp( cmdBuffer, "deldir ", strlen("deldir ") ) == 0  && 
		argc( cmdBuffer ) == 2 )
	{
		clientData.type = CM_DELDIR;
		bzero( clientData.data, sizeof(clientData.data) );

		char filename[DATASIZ] = "";
		sscanf( &cmdBuffer[7], "%s", filename );
		sprintf( clientData.data, "%s", filename );
		clientData.len  = strlen( clientData.data );
		return sendCmd();
	}

	// 파일 삭제
	if( strncmp( cmdBuffer, "delfile ", strlen("delfile ") ) == 0  && 
		argc( cmdBuffer ) == 2 )
	{
		clientData.type = CM_DELFILE;
		bzero( clientData.data, sizeof(clientData.data) );

		char filename[DATASIZ] = "";
		sscanf( &cmdBuffer[8], "%s", filename );
		sprintf( clientData.data, "%s", filename );
		clientData.len  = strlen( clientData.data );
		return sendCmd();
	}

	// 파일 추가
	if( strncmp( cmdBuffer, "addfile ", strlen("addfile ") ) == 0  && 
		argc( cmdBuffer ) == 3 )
	{
		clientData.type = CM_ADDFILE;
		bzero( clientData.data, sizeof(clientData.data) );

		char filename[DATASIZ] = "";
		bzero( md5buf, sizeof(md5buf) );
		sscanf( &cmdBuffer[8], "%s %s", filename, md5buf);
		sprintf( clientData.data, "%s %s", filename, md5buf );
		clientData.len  = strlen( clientData.data );
		/*
		printf("[%d][%d][%s]",
				clientData.type,
				clientData.len,
				clientData.data );
				*/
		switch( sendCmd() )
		{
			case CM_ACKOK :
				// 파일을 전송해야 할 부분
				return CM_ACKOK;
			case CM_MD5HASH :
				// 파일의 해쉬코드가 같음 전송 필요 없음
				return CM_MD5HASH;
			default :
				// 정책상 밀리는 파일일 경우
				return CM_ACKNO;
		}
	}

	// SM으로 보낼 루틴
	if( strncmp( cmdBuffer, "add ", strlen("add ") ) == 0  && 
		argc( cmdBuffer ) == 5 || argc( cmdBuffer ) == 6 )
	{
		if( addSync( &cmdBuffer[4] ) )
			return CM_ACKOK;
		else
			return CM_ACKNO;
	}

	if( strncmp( cmdBuffer, "del ", strlen("del ") ) == 0  && 
		argc( cmdBuffer ) == 2 || argc( cmdBuffer ) == 3 )
	{
		if( delSync( &cmdBuffer[4] ) )
			return CM_ACKOK;
		else
			return CM_ACKNO;
	}

	if( strncmp( cmdBuffer, "update ", strlen("update ") ) == 0  && 
		argc( cmdBuffer ) == 2 )
	{
	//	sm->updateSyncTime( &cmdBuffer[7], lockTime );
		sm->updateSyncTime( &cmdBuffer[7], time(NULL) );
		return CM_ACKOK;
	}

	setcolor( COLOR_SKY );
	cout << "parsing::[" << cmdBuffer << "]" << endl;
	setcolor( COLOR_NONE ); 

	return CM_ACKOK;
}

/*
 * 함수 : 동기화를 삭제한다. 
 */
int CommunicationManager::delSync( char *cmd )
{
	if( argc( cmd ) == 1 )
	{
		return sm->delSyncNode( cmd );
	}
	else
	{
		return sm->delSyncNodeFromRemote( strtok( cmd, " " ) );
	}
}

/*
 * 함수 : 동기화를 추가한다.
 */
int CommunicationManager::addSync( char *cmd )
{
	char name[SDATA]  	= "";
	char target[SDATA] 	= "";
	char lpath[SDATA]  	= "";
	char rpath[SDATA]  	= "";
	char id[SDATA]     	= "id";
	char pass[SDATA] 	= "pass";
	int  filesize  		= 10000000;
	int  flag			= FLAG_NEWSYNC;

	if( argc( cmd ) == 4 )
	{
		sscanf( cmd, "%s %s %s %s", name, target, lpath, rpath );
	}
	else 
	{
		sscanf( cmd, "%s %s %s %s %d", name, target, lpath, rpath, &flag );
		bzero( target, sizeof(target) );
		strcpy( target, inet_ntoa( client_addr.sin_addr ) );
	}

	return sm->addSync( name, target, lpath, rpath, id, pass , filesize, flag );
}

/*
 * 함수 : 초기화 함수
 */
void CommunicationManager::init( void )
{
	bzero( remoteTarget, sizeof(remoteTarget) );
	bzero( buf, sizeof(buf) );
	bzero( cmdBuffer, sizeof(cmdBuffer) );

	listenPort 	= CM_PORT;
	sendPort	= CM_PORT;
	controlPort	= CONTROL_PORT;

	isLock 		= false;
	debugMode	= false;
	demon		= false;

	client_len 	= sizeof(client_addr);
    server_len 	= sizeof(server_addr);
	rtarget_len = sizeof(rtarget_addr);

	bzero( username, sizeof(username) );
	strncpy( username, getlogin(), sizeof(username) );

	bzero( recordLockPath, sizeof(recordLockPath) );
	strncpy( recordLockPath, CM_RLOCK_PATH, sizeof(recordLockPath) );
	strncat( recordLockPath, username, sizeof(recordLockPath) - strlen(username) - 1 );

	memset( &serverData, 0, sizeof(serverData) );
	memset( &serverResult, 0, sizeof(serverResult) );
	memset( &clientData, 0, sizeof(clientData) );
	memset( &clientResult, 0, sizeof(clientResult) );

	fdRecordLock = -1;

	// 프로그램 중복실행 방지
	if( !recordLock() )
	{
		exit(0);
	}

//	setPassword();

	// 디버깅을 위한 타겟 로컬지정
	strcpy( remoteTarget, "127.0.0.1" );

	return;
}

/*
 * 함수 : 패스워드 입력받음
 */
void CommunicationManager::setPassword( void )
{
	char buf1[SDATA], buf2[SDATA];

	bzero( buf1, sizeof(buf1) );
	bzero( buf2, sizeof(buf2) );

	while( 1 )
	{
		echoOFF();
		cout << "프로그램의 패스워드를 지정합니다." << endl;
		cout << "Password > ";
		fgets( buf1, SDATA, stdin );
		cout << endl << "Retype > ";
		fgets( buf2, SDATA, stdin );
		cout << endl;
		echoON();

		if( strlen( buf1 ) != strlen( buf2 ) ||
				strncmp( buf1, buf2, strlen( buf1 ) ) != 0 )
		{
			cout << "패스워드가 동일하지 않습니다." << endl;
			continue;
		}

		if( strlen( buf1 ) <= CM_PASS_LEN )
		{
			cout << "패스워드가 너무 짧습니다. 기본 " <<
				     CM_PASS_LEN << "글자 이상" << endl; 
			continue;
		}

		// '\n' 제거
		buf1[strlen(buf1)-1] = '\0';
		bzero( userpass, sizeof(userpass) );
		strncpy( userpass, buf1, sizeof(userpass) );
		break;
	}
	return;
}

/*
 * 함수 : 패스워드를 출력
 */
char *CommunicationManager::getPassword( void )
{
//	cout << userpass << endl;
	return userpass;
}

/*
 * 함수 : 터미널 에코 OFF
 */
void CommunicationManager::echoOFF( void )
{
	struct termios new_settings;

	tcgetattr( 0, &termSettings );

	new_settings = termSettings;
	new_settings.c_lflag &= (~ECHO);
	new_settings.c_lflag &= (~ICANON);

	tcsetattr( 0, TCSANOW, &new_settings );

	return ;
}

/*
 * 함수 : 터미널 에코 ON
 */
void CommunicationManager::echoON( void )
{
	tcsetattr( 0, TCSANOW, &termSettings );

	return ;
}

/*
 * 함수 : 메모리 정리 함수
 */
void CommunicationManager::cleanUp( void )
{
	recordUnlock();

	return;
}

/*
 * 함수 : 컨트롤 서버를 초기화 한다.
 */
void CommunicationManager::init_control_server( void )
{
    if( (control_sockfd = socket(PF_INET, SOCK_STREAM, 0)) < 0 )
    {
        perror("init_server() socket error : ");

		return;
    }

	bzero( &control_addr, sizeof(control_addr) );
    server_addr.sin_family = AF_INET;
    server_addr.sin_addr.s_addr = htonl(INADDR_ANY);		// 자신의 IP 사용
    server_addr.sin_port = htons( controlPort );

	return;
}

/*
 * 함수 : 소켓 통신을 위한 서버 초기화
 * 설명 : 기초적인 서버의 정보를 초기화 한다.
 */
void CommunicationManager::init_server( void )
{
    if( (server_sockfd = socket(PF_INET, SOCK_STREAM, 0)) < 0 )
    {
        perror("init_server() socket error : ");

		return;
    }

	bzero( &server_addr, sizeof(server_addr) );
    server_addr.sin_family = AF_INET;
    server_addr.sin_addr.s_addr = htonl(INADDR_ANY);		// 자신의 IP 사용
    server_addr.sin_port = htons( listenPort );

	return;
}

/*
 * 함수 : 상위포트로 이동
 * 설명 : 현재 listen중인 포트보다 상위포트가 비어있으면 할당
 */
void CommunicationManager::portRefresh( void )
{
	int tmp_sockfd;

	// 현재 프로그램이 최 상위포트 이면 그냥 종료
	if( listenPort == CM_PORT )
	{
		return;
	}

	// 프로그램이 최상위 포트가 아닐 경우 
	for( int i = CM_PORT; i < listenPort; i++ )
	{
		if( (tmp_sockfd = socket(PF_INET, SOCK_STREAM, 0)) < 0 )
		{
			perror("init_server() socket error : ");

			return;
		}

		bzero( &server_addr, sizeof(server_addr) );
		server_addr.sin_family = AF_INET;
		server_addr.sin_addr.s_addr = htonl(INADDR_ANY);		// 자신의 IP 사용
		server_addr.sin_port = htons( listenPort );

	}
	return;
}

/*
 * 함수 : 서버 소켓의 주소 할당
 * 설명 : 기본 포트 이후로 검색하며 비어 있는 포트에 주소를 할당
 *        할당된 포트는 멤버변수 listenPort에 저장
 */
void CommunicationManager::bind_server( void )
{
    int state = -1;

	// 포트 할당
	while( state < 0 )
    {
		server_addr.sin_port = htons( listenPort );
		state = bind( server_sockfd, (struct sockaddr *)&server_addr, server_len );

		if( state < 0 )
		{
			if( errno == EADDRINUSE )
			{
				setcolor( COLOR_RED );
				cout << listenPort << " 포트 사용중" << endl;
				cout << "새로운 포트 할당 시도중.." << endl;
				setcolor( COLOR_NONE );

				listenPort++;

				if( listenPort > CM_PORT_END )
				{
					cout << "CM::bind_server() - 포트 할당 실패." << endl;
					exit(0);
				}

				continue;
			}

			perror("bind_server() bind error : ");
		}

		// 포트 할당 성공
		if( state >= 0 )
		{
			cout << "[1;36m";
			cout << "포트번호 : " << listenPort << endl;
			cout << "[00m";
			break;
		}
    }
	
    state = listen( server_sockfd, 10 );
    if( state == -1 )
    {
        perror("bind_server() listen error : ");
        exit(0);
    }

	return;
}

/*
 * 함수 : 소켓 통신을 위한 클라이언트 초기화
 * 인자 : 없음
 * 반환 : 성공 : 0, 실패 : 1
 */
int CommunicationManager::init_client( void )
{
	rtarget_addr.sin_family = AF_INET;
	rtarget_addr.sin_port = htons( sendPort );

	// 원격지 주소가 IP일 경우
	if( isIPformat( remoteTarget ) )
	{
		rtarget_addr.sin_addr.s_addr = inet_addr( remoteTarget );
	}
	// 원격지 주소가 domain name 일 경우
	else
	{
		struct hostent *he;
		he = gethostbyname( remoteTarget );

		// gethostbyname 결과 IP를 받아오지 못했을 경우
		if( he == NULL )
		{
			return 1;
		}
		else
		{
			rtarget_addr.sin_addr.s_addr = inet_addr( inet_ntoa( *(struct in_addr*)he->h_addr) );
		}
	}

	return 0;
}

/*
 * 함수 : 소켓 통신을 위한 클라이언트 초기화
 * 인자 : 없음
 * 반환 : 성공 : 0, 실패 : -1
 */
int CommunicationManager::connect_client( void )
{
    if( (rtarget_sockfd = socket(PF_INET, SOCK_STREAM, 0)) < 0 )
    {
		if( debugMode ) puts("connect_client() socket() failed");
		return -1;
    }

    if( connect(rtarget_sockfd, (struct sockaddr *)&rtarget_addr, rtarget_len) < 0)
    {
		if( debugMode ) puts("connect_client() connect() failed");
		if( rtarget_sockfd )
		{
			close( rtarget_sockfd );
		}
		return -1;
    }

	return 0;
}

/*
 * 함수 : buf 에 있는 내용을 원격지로 전송한다.
 * 인자 : 없음
 * 반환 : 성공 : 0, 실패 : -1
 */
int CommunicationManager::sendCmd( void )
{
	if( rtarget_sockfd <= 0 )
	{
		if( debugMode ) printf("rserver_sockfd is not valid.\n");
		sendResult = CM_ACKNO;
		return CM_ACKNO;
	}

	// 전송을 정해진 양만큼만 보냄
	write( rtarget_sockfd, &clientData, 
		   sizeof(clientData.type) + sizeof(clientData.len) + strlen(clientData.data) );
	if( debugMode )
	{
		setcolor( COLOR_GREEN );
		printf("[Send] [%d] [%d] [%s] -- ", 
				clientData.type, clientData.len, clientData.data  );
		setcolor( COLOR_NONE );
	}

	// 결과값 수신
	memset( &clientResult, 0, sizeof( clientData ) );
	read( rtarget_sockfd, &clientResult, sizeof(clientResult.type) + sizeof(clientResult.len) );
	if( clientResult.len > 0 )
	{
		read( rtarget_sockfd, &clientResult.data, clientResult.len );
	}

	if( debugMode )
	{
		setcolor( COLOR_SKY );
		printf("[Receive] [%d] [%d] [%s]\n", 
				clientResult.type, clientResult.len, clientResult.data  );
		setcolor( COLOR_NONE );
	}

	return clientResult.type;
}

/*
 * 함수 : buf 에 있는 내용으로 대답한다.
 * 인자 : 없음
 * 반환 : 성공 : 0, 실패 : -1
 */
int CommunicationManager::reply( void )
{
	/*
	 * 현재 구현 불가.
	 * 서버에서 전송받는 부분과 메시지를 전송하는 부분이 분리되어 있음
	 */
	write( rtarget_sockfd, &clientData, sizeof(clientData.type) + sizeof(clientData.len) );
	write( rtarget_sockfd, &clientData.data, strlen(clientData.data) );
	printf("\n[Reply Send] [%d] [%d] [%s]\n", 
			clientData.type, clientData.len, clientData.data  );

	memset( &clientResult, 0, sizeof( clientData ) );
	read( rtarget_sockfd, &clientResult, sizeof(clientResult.type) + sizeof(clientResult.len) );
	if( clientResult.len > 0 )
	{
		read( rtarget_sockfd, &clientResult.data, clientResult.len );
	}
	printf("\n[Receive] [%d] [%d] [%s]\n", 
			clientResult.type, clientResult.len, clientResult.data  );

	return 0;
}

/*
 * 함수 : 원격지와의 연결을 종료한다.
 * 인자 : 없음
 * 반환 : 성공 : 0, 실패 : 1
 */
int CommunicationManager::disconnect_client( void )
{
	close( rtarget_sockfd );

	return 0;
}

/*
 * Thread Function ===================================
 * 함수 : 타이머 구현 함수
 * 설명 : 주기적으로 실행할 함수를 등록한다.
 */
void *CommunicationManager::timer( void *pthis )
{
	CommunicationManager *cm = (CommunicationManager *)pthis;

	while( 1 )
	{
		sleep( 1 );

		if( cm->isLock )
		{
			continue;
		}

		cm->portRefresh();
	}

	pthread_exit(NULL);
	return 0;
}

/*
 * Thread Function ===================================
 * 함수 : 클라이언트의 접속을 처리한다.
 * 설명 : 서버의 기능으로 접속을 기다리며
 * 		  새로운 클라이언트가 접속하면 스레드를 생성하여
 * 		  명령을 처리한다.
 */
void *CommunicationManager::accept_server( void *pthis )
{
	CommunicationManager *cm = (CommunicationManager *)pthis;

	// 다음 접속을 기다린다.
	while( 1 )
	{
		cm->client_sockfd = accept( cm->server_sockfd, 
				(struct sockaddr *)&cm->client_addr, 
				&cm->client_len );
		if( cm->client_sockfd == -1 )
		{
			perror("clientConnect() Accept error : ");
			return 0;
		}

		cm->status_cs = pthread_create( &cm->cm_tid, 
									NULL, 
									CommunicationManager::clientConnect,
									cm );
	}

	return 0;
}

/*
 * Thread Function ===================================
 * 함수 : 클라이언트의 접속을 처리한다.
 * 설명 : 서버의 기능으로 접속을 기다리며
 * 		  접속된 클라이언트로부터 명령을 받아 처리한다
 * 		  더이상 받을 메시지가 없으면 종료
 */
void *CommunicationManager::clientConnect( void *pthis )
{
	CommunicationManager *cm = (CommunicationManager *)pthis;

	int clientSockFd = cm->client_sockfd;
	clientInfo cltmp;

	// 스레드의 TID를 얻어오는 함수 찾아서 집어 넣을 것.
	cltmp.tid = cm->cm_tid;
	cltmp.clientSockFd = clientSockFd;
	cltmp.clientIP = inet_ntoa( cm->client_addr.sin_addr );

	// 접속중인 클라이언트 리스트에 삽입
//	cm->clientList.push_back( cltmp );

	if( cm->debugMode )
	{
		setcolor( COLOR_YELLOW );
		cout << "\nClient Connected::sockfd = " << clientSockFd;
		cout << " (form " << cltmp.clientIP << ")" << endl;
		setcolor( COLOR_NONE );
	}

	int len;
	// 명령을 읽어온다
	while( 1 )
	{
		memset( &cm->serverData, 0, sizeof(cm->serverData) );
		len = read( clientSockFd, &cm->serverData,
				sizeof(cm->serverData.type) + sizeof(cm->serverData.len) );
		if( cm->serverData.len > 0 )
		{
			read( clientSockFd, &cm->serverData.data, cm->serverData.len );
		}

		if( (cm->serverData.type == CM_LOCK) && cm->isLock )
		{
			write( clientSockFd, 
					cm->makeCmd(CM_ACKNO, "locking"),
					4 + 4 + strlen("locking..") );
		}

		if( len <= 0 )
		{
			close( clientSockFd );

			if( cm->debugMode )
			{
				setcolor( COLOR_YELLOW );
				cout << "\n(" << len << ")DisConnected::sockfd = " << clientSockFd << endl;
				setcolor( COLOR_NONE );
			}

			break;
		}

		//////////////////////////////////////////////////
		// 읽어온 명령을 해석한다
		////////////////////////////////////////////////// 
		char filename[SDATA] = "";
		char hashcode[SDATA] = "";
		if( cm->debugMode )
		{
			cout << "(" << clientSockFd << ")receive :: " << cm->serverData.data << endl;
		}
		switch( cm->serverData.type )
		{
			case CM_ADDDIR :	// 디렉토리 생성
				mode_t perm;
				sscanf( cm->serverData.data, "%s %d", filename, &perm );
				if( access( filename, F_OK|W_OK ) )
				{
					mkdir( filename, perm );
				}
				cm->receiveResult = CM_ACKOK;
				break;
			case CM_DELDIR :	// 디렉토리 삭제
				sscanf( cm->serverData.data, "%s", filename );
				if( !access( filename, F_OK|W_OK ) )
				{
					rmdir( filename );
				}
				cm->receiveResult = CM_ACKOK;
				break;
			case CM_ADDFILE :	// 파일 생성
				sscanf( cm->serverData.data, "%s %s", filename, hashcode );
				if( !access( filename, F_OK|W_OK ) )
				{
					if( strcmp( md5file(filename), hashcode) == 0 )
					{
						// Hashcode 가 같다.
						cm->receiveResult = CM_MD5HASH;
					}
				}
				else
				{
					if( 0 )
					{	// 내가 우선순위에 앞서는 파일을 소유중일 때
						cm->receiveResult = CM_ACKNO;
					}
					else
					{	// 내가 우선쉰위에 밀리는 파일을 소유중일 때
						cm->receiveResult = CM_ACKOK;

						memset( &cm->serverResult, 0, sizeof( cm->serverResult ) );
						cm->serverResult.type = cm->receiveResult;
						cm->serverResult.len = 0;
						write( clientSockFd, &cm->serverResult, 
								sizeof(cm->serverData.type) + sizeof(cm->serverData.len) );

						// CM을 이용한 파일 수신
						cm->receiveResult = cm->receiveFile();
						break;
					}
				}
				break;
			case CM_DELFILE :	// 파일 삭제
				sscanf( cm->serverData.data, "%s", filename );

				if( 0 )
				{
					// 내가 우선순위에 앞서는 파일을 소유중일 때
					cm->receiveResult = CM_ACKNO;
				}
				else
				{
					// 내가 우선순위에 밀리는 파일을 소유중일 때
					if( !access( filename, F_OK ) && remove( filename ) )
					{
						if( cm->debugMode )
						{
							cout << "CM_DELFILE :: 파일 삭제 실패" << endl;
						}
						cm->receiveResult = CM_ACKNO;
					}
					else
					{
						cm->receiveResult = CM_ACKOK;
					}
				}
				break;
			case CM_UPDATESYNCTIME :
			case CM_ADDSYNC :
			case CM_DELSYNC :
			case CM_UNLOCK :
			case CM_LOCK :
			case CM_TEST :
			default :
				strncpy( cm->cmdBuffer, cm->serverData.data, sizeof(cm->cmdBuffer) );
				cm->receiveResult = cm->receiveDataParsing();
				break;
		}

		// 실행 결과 송출
		memset( &cm->serverResult, 0, sizeof( cm->serverResult ) );
		cm->serverResult.type = cm->receiveResult;
		cm->serverResult.len = 0;
		write( clientSockFd, &cm->serverResult, 
				sizeof(cm->serverData.type) + sizeof(cm->serverData.len) );
	}

	return 0;
}

/*
 * 함수 : 커맨드를 만들어 리턴한다.
 * 설명 : 스태틱 변수를 사용하므로 일회성 커맨드임을 명심!!
 */
char *CommunicationManager::makeCmd( int type )
{
	return makeCmd( type, NULL );
}
char *CommunicationManager::makeCmd( int type, char *data )
{
	static char buf[sizeof(cmdData)];
	int len = strlen(data);
	strncpy( buf, data, len );

	memcpy( &buf[0], &type, sizeof(int) );
	memcpy( &buf[4], &len, sizeof(int) );
	memcpy( &buf[8], data, len );

	return buf;
}

/*
 * 함수 : 클래스 내부 변수들 확인
 */
void CommunicationManager::debug( void )
{
	list<clientInfo>::iterator clIndex;

	setcolor( COLOR_SKY );
	cout << "[=========================== ACHA  Infomation ============================]\n";
	setcolor( COLOR_NONE );
	cout << "[1m";
	cout << " UserName / password     : " << username << " / " << userpass << endl;
	cout << " RecordLockPath          : " << recordLockPath << "(" << fdRecordLock << ")\n";
	cout << " Current Remote target   : " << remoteTarget << endl;
	cout << " server_sockfd           : " << server_sockfd << endl;
/*
	cout << " Connected Client List   : " << clientList.size() << endl;
	for( clIndex = clientList.begin(); clIndex != clientList.end(); clIndex++ )
	{
		cout << "  [IP : " << clIndex->clientIP << "] [TID : ";
		cout << clIndex->tid << "] [SOCKFD : ";
		cout << clIndex->clientSockFd << "]" << endl;
	}
*/
	cout << " rtarget_sockfd          : " << rtarget_sockfd << endl;

	cout << " Listen / Send port      : " << listenPort << " / " << sendPort << endl;
	cout << " CM Lock / Debug  state  : " << isLock << " / " << debugMode << endl;
	cout << " SM Lock / Debug  state  : " << sm->lockState << " / " << sm->debugMode << endl;
	cout << " FM Lock / Debug  state  : " << fm->lockState << " / " << fm->debugMode << endl;
	cout << " LastClient IP           : " << inet_ntoa( client_addr.sin_addr ) << endl;
	cout << " LastRemote IP           : " << inet_ntoa( rtarget_addr.sin_addr ) << endl;
	cout << " CmdBuffer               : " << cmdBuffer << endl;
	cout << " serverData              : ";
	printf( "[%d] [%d] [%s]\n", serverData.type, serverData.len, serverData.data );
	cout << " clientData              : ";
	printf( "[%d] [%d] [%s]\n", clientData.type, clientData.len, clientData.data );
	cout << "[00m";

	sm->showList();
	fm->showList(SHOW_CHANGE);

	return;
}

/*
 * 함수 : 파일을 전송한다.
 * 인자 : lPath : 로컬의 전송대상 파일
 *        rPath : 원격지에 생성될 파일
 *        hashcode : 로컬 파일의 md5 해쉬코드
 * 반환 : CM_
 */
int CommunicationManager::sendFile( char *lPath, char *rPath, char *hashcode )
{
	int fd;
	int buffer[DATASIZ];
	struct stat status;
	mode_t filePermission;

	fd = open( lPath, O_RDONLY );
	if (fd < 0)
	{
		cout << "CM::sendFile() - Can't Open File" << endl;
		return CM_ERROR;
	}
	fstat(fd, &status);

	totalSize 		= status.st_size;
	// ## linux version
	filePermission 	= status.st_mode;
	// ## windows version
	// filePermission 	= status.st_mode;

	// 기본정보 전송
	memset( &clientData, 0, sizeof(clientData) );
	clientData.type = TM_INFO;
	sprintf( clientData.data, "%s|%d|%s|%d", rPath, totalSize, hashcode, filePermission );
	clientData.len = strlen( clientData.data );

	write( rtarget_sockfd, &clientData, 
			sizeof(clientData.type) + sizeof(clientData.len) + strlen(clientData.data) );
	//	printf("[TM::Send Data] [%d] [%d] [%s]\n", 
	//			clientData.type, clientData.len, clientData.data  );

	if( debugMode )
	{
		printf("CM::local[%s] => %s[%s] 전송중..", lPath, remoteTarget, rPath);
	}

	// 데이터 전송
	int len;
	for( int amount = 0; amount < totalSize; )
	{
		memset( buffer, 0, sizeof(buffer) );

		len = read( fd, buffer, DATASIZ );
		if( len <= 0 )
		{
			break;
		}

		write( rtarget_sockfd, buffer, len );
		amount += len;

		sendAmount = amount;
	}

	close( fd );

	// 전송 결과 확인
	memset( &serverData, 0, sizeof(serverData) );
	read( rtarget_sockfd, &serverData, sizeof(serverData.type) + sizeof(serverData.len) );
	read( rtarget_sockfd, serverData.data, serverData.len );

	if( serverData.type == CM_ACKOK )
	{
		if( debugMode )
		{
			setcolor( COLOR_PINK );
			cout << " 완료!!" << endl;
			setcolor( COLOR_NONE );
		}
		return 0;
	}
	else
	{
		if( debugMode ) cout << " 실패" << endl;
		return serverData.type;
	}
	if( debugMode )
	{
		printf( "전송 결과 : [%d] [%d] [%s]\n", serverData.type, serverData.len, serverData.data );
	}
}

/*
 * 함수 : 파일을 전송 받는다
 * 반환 : 성공 : CM_ACKOK
 *        실패 : CM_ACKNO
 */
int CommunicationManager::receiveFile( void )
{
	char filePath[DATASIZ] = "";
	char hashCode[DATASIZ] = "";
	mode_t filePermission;

	// 전송 받을 파일에 관한 정보를 먼저 수신한다.
	memset( &serverData, 0, sizeof(serverData) );
	read( client_sockfd, &serverData,
			sizeof(serverData.type) + sizeof(serverData.len) );
	if( serverData.len > 0 )
	{
		read( client_sockfd, &serverData.data, serverData.len );
	}

//	sscanf( tm->serverData.data, "%s|%d|%s|%d", 
//			tm->filePath, &tm->totalSize, tm->hashCode, &filePermission );

	// 수신한 데이터를 파싱한다.
	char dataBuf[DATASIZ] = "";
	strncpy( dataBuf, serverData.data, DATASIZ );
	strcpy( filePath, strtok( dataBuf, "|" ) );
	totalSize = atoi( strtok( NULL, "|") );
	strcpy( hashCode, strtok( NULL, "|" ) );
	filePermission = (mode_t)atoi( strtok( NULL, "|\0") );

//	setcolor( COLOR_SKY );
//	printf( "[TM::receive] [%s] [%d] [%s] [%d]\n", 
//			tm->filePath, tm->totalSize, tm->hashCode, filePermission );
//	setcolor( COLOR_NONE );

	// Temp 폴더가 없으면 만든다.
	if( access( TEMPFOLDER, F_OK | W_OK ) )
	{
		mkdir( TEMPFOLDER, (mode_t)0777 );
	}

	fpTemp = fopen( TMTEMP, "w" );
	if( fpTemp == NULL )
	{
		if( debugMode )
		{
			cout << "TM::clientConnect() - tmp 파일 생성 실패" << endl;
		}
		return CM_ACKNO;
	}

	if( debugMode )
	{
		printf("[%s] 수신중..", filePath);
	}
	// 명령을 읽어온다
	char buffer[DATASIZ];
	int len;
	for( int amount = 0; amount < totalSize; )
	{
		memset( buffer, 0, sizeof(buffer) );
		len = read( client_sockfd, buffer, DATASIZ );
		if( len <= 0 )
		{
			break;
		}

		fwrite( buffer, len, 1, fpTemp );
		amount += len;

		receiveAmount = amount;
	}

	fclose( fpTemp );

	if( strcmp( hashCode, md5file( TMTEMP ) ) == 0 )
	{
		if( debugMode )
		{
			setcolor( COLOR_SKY );
			cout << " MD5 check.";
			setcolor( COLOR_NONE );
		}
	}
	else
	{
		if( debugMode )
		{
			cout << "MD5 체크/파일 수신 실패" << endl;
		}
		return CM_ACKNO;
	}

	// temp 파일을 기본 파일로 변경
	if( rename( TMTEMP, filePath ) )
	{
		if( debugMode )
		{
			printf( "파일 경로 변경 : %s -> %s -- 실패\n", TMTEMP, filePath );
		}
		return CM_ACKNO;
	}
	if( debugMode )
	{
		setcolor( COLOR_GREEN );
		cout << " 파일생성";
		setcolor( COLOR_NONE );
	}

	// 파일 퍼미션 변경
	if( chmod( filePath, filePermission ) )
	{
		if( debugMode )
		{
			cout << "파일 권한 변경 -- 실패" << endl;
		}
		return CM_ACKNO;
	}
	if( debugMode )
	{
		setcolor( COLOR_YELLOW );
		cout << " 권한변경";
		setcolor( COLOR_NONE );
	}

	/*
	   memset( &serverData, 0, sizeof(serverData) );
	   serverData.type = TM_OK;
	   serverData.len  = 0;
	   write( client_sockfd, &serverData,
	   sizeof(serverData.type) + sizeof(serverData.len) + serverData.len );
	   */

	if( debugMode )
	{
		setcolor( COLOR_PINK );
		cout << " 완료!!" << endl;
		setcolor( COLOR_NONE );
	}

	if( debugMode )	cout << endl;

	return CM_ACKOK;
}

