#include "acha.h"
#include "sm.h"
#include "cm.h"
#include "fm.h"
#include "tm.h"

#ifdef DEBUG
int main(int argc, char* argv[])
{
	SyncManager *sm = new SyncManager();
	sm->debugMode = 1;

	//sm->loadSyncList();
	sm->addSync("mysync", "203.253.22.23", "./test", "./test", "korzero", "1111", 1024, 1);

	sm->debug();

	sm->addSync("mysync1", "203.253.22.23", "./test", "./test", "korzero", "1111", 1024, 1);

	sm->debug();
	
	sm->addSync("mysync2", "203.253.22.23", "./test", "./test", "korzero", "1111", 1024, 1);

	sm->debug();

	sm->delSyncNode("mysync1");

	sm->debug();

	sm->test();

	printf("change Time\n");

	sm->updateSyncTime("mysync2", (time_t)345);

	sm->debug();

	sm->run();

	return 0;
}
#endif

void SyncManager::test( void )
{
	cout << "compile ok!!" << endl;
}

/************************************
 * 함수 : SyncList(File)를 로드하는 함수
 * 설명 : synclist 파일의 형식 
 *        SuncName | IP or Domain | localPath | remotePath |
 *        userId | password | flag | lastSyncTime
 *        위와 같은 형식의 파일을 읽어 syncList(Memory) 에 저장한다.
 * 참고 : syncList( STL의 dequeue형 )
 * ************************************/
void SyncManager::loadSyncList( void )
{
    if( access( CONFIGDIR, F_OK | W_OK ) ) 			// 폴더존재여부 확인
    {   
        mkdir( CONFIGDIR, (mode_t)0777 );			// 없으면 폴더생성
    }   

    char buf[DATASIZ];

    fpSyncList = fopen( SYNCLISTPATH, "r+" );		// 파일 open

	if( fpSyncList == NULL )						// 기존Sync규칙이 없으면
    {   
        puts("Sync 규칙이 만들어지지 않았습니다.");
        puts("새로운 SyncList 파일을 생성합니다.");
        fpSyncList = fopen( SYNCLISTPATH, "w" );	// 파일 생성후 종료
        fclose( fpSyncList );
        return;
    }   

    cleanSyncList();								// SyncList 메모리 초기화

    while( 1 ) 
    {   
        char *tok;
        bzero( buf, sizeof(buf) );						// 버퍼를 '\0'으로 초기화
        fgets( buf, DATASIZ, fpSyncList );			// 파일로부터 버퍼만큼 읽어

        if( !strlen(buf) )							// 읽은 문자가 없으면
        {
            break;									// 종료
        }

        tok = strtok( buf, "|" );					// SyncList File parsing
        slnTmp.syncName = strdup( tok );
        tok = strtok( NULL, "|" );
        slnTmp.target = strdup( tok );
        tok = strtok( NULL, "|" );
        slnTmp.localPath = strdup( tok );
        tok = strtok( NULL, "|" );
        slnTmp.remotePath = strdup( tok );
        tok = strtok( NULL, "|" );
        slnTmp.userId = strdup( tok );
		tok = strtok( NULL, "|" );
		slnTmp.password = strdup( tok );
		tok = strtok( NULL, "|" );
		sscanf( tok, "%d", &slnTmp.fileSize );
		tok = strtok( NULL, "|" );
		sscanf( tok, "%d", &slnTmp.option );
		tok = strtok( NULL, "|" );
        sscanf( tok, "%d", &slnTmp.Flag );

        int ttmp;
        tok = strtok( NULL, "|\n" );
        sscanf( tok, "%d", &ttmp );					// 문자 -> int형 변수
        
		slnTmp.lastSyncTime = (time_t)ttmp;

        if( debugMode )
        {
            printf("[syncList] : ");
            printf("%s|%s|%s|%s|%s|%s|%d|%d|%d|%s|\n",
                    slnTmp.syncName, slnTmp.target,
                    slnTmp.localPath, slnTmp.remotePath,
					slnTmp.userId, slnTmp.password,
					slnTmp.fileSize, slnTmp.option,
					slnTmp.Flag, 
					timeFormat(slnTmp.lastSyncTime) );
        }
        syncList.push_back( slnTmp );				// SyncList에 추가
    }

    fclose( fpSyncList );

    return;
}

/* 함수 : syncList를 초기화 하는 함수
 * 설명 : dequeue에 있는 syncListNode를 가져와서 메모리를 해제한 후
 * 	      dequeue에 있는 내용을 pop 한다.
 */
void SyncManager::cleanSyncList( void )
{
    syncListNode tmp;
    while( !syncList.empty() )	// syncList에 있는 모든 내용 삭제
    {
        tmp = syncList.front();
		cleanSyncListNode(tmp);
        syncList.pop_front();
    }

    return;
}

/* 함수 : syncListNode한개의 메모리를 해제하는 함수
 * 설명 : 동적으로 할당된 메모리를 반환한다.
 */
void SyncManager::cleanSyncListNode( syncListNode tmp )
{
	delete tmp.syncName;
    delete tmp.target;
    delete tmp.localPath;
    delete tmp.remotePath;
	delete tmp.userId;
	delete tmp.password;
}

/* 함수 : syncList(Memory)에 있는 내용을 파일로 쓰는 함수
 * 주의 : 기존에 파일이 존재한다면 기존의 파일은 삭제되고 새롭게 생성된다.
 */
void SyncManager::writeSyncList( void )
{
    if( !access( CONFIGDIR, F_OK | W_OK ) )	// config dir check 및 생성
    {
        mkdir( CONFIGDIR, (mode_t)0777 );
    }

    fpSyncList = fopen( SYNCLISTPATH, "w" );// 파일이 존재하면 기존의 파일은 삭제
    if( fpSyncList == NULL )
    {
        puts("writeSyncList() SyncList 파일을 쓸 수 없습니다.");
        return;
    }

    for( int i = 0; i < (int)syncList.size(); i++ )
    {
        fprintf( fpSyncList, "%s|%s|%s|%s|%s|%s|%d|%d|%d|%d|\n",
                syncList[i].syncName,
                syncList[i].target,
                syncList[i].localPath,
                syncList[i].remotePath,
				syncList[i].userId,
				syncList[i].password,
				syncList[i].fileSize,
				syncList[i].option,
                syncList[i].Flag,
                (int)syncList[i].lastSyncTime);

    }

    fclose( fpSyncList );
    return;
}

/* 함수 : 해당 이름의 Sync가 존재하는 지 확인
 * 반환 : 존재 index, 없음 0
 */
int SyncManager::syncExist( char *name )
{
    for( unsigned int i = 0; i < syncList.size(); i++ )
    {
        if( strcmp( syncList[i].syncName, name ) == 0 )
        {
            return i + 1;
        }
    }
    return 0;
}


/* 함수 : 새로운 sync를  생성하는 함수
 *
 */
int SyncManager::addSync( char *syncname, char *target, char *localpath, char *remotepath, char *userId, char *password , int filesize, int flag)
{
    if( syncExist( syncname ) )
    {
        printf("SM::addSync() - Sync생성 실패 : %s 이름의 싱크가 존재합니다.\n", syncname);
        return 0;
    }
	
	if( access( localpath, F_OK | R_OK ) )
	{
		cout << "SM::addSync() - Sync생성 실패 : 존재하지 않는 디렉토리" << endl;
		return 0;
	}

    slnTmp.syncName     = strdup( syncname );
    slnTmp.target       = strdup( target );
    slnTmp.localPath    = strdup( localpath );
    slnTmp.remotePath   = strdup( remotepath );
	slnTmp.userId		= strdup( userId );
	slnTmp.password		= strdup( password );
	slnTmp.fileSize		= filesize;
	slnTmp.option		= 0;

    slnTmp.Flag 		= flag;
//    slnTmp.lastSyncTime = time( NULL );				// 확인 요망
    slnTmp.lastSyncTime = 0;				// 확인 요망

    syncList.push_back( slnTmp );

	// Sync name 의 lastList 가 들어갈 파일을 생성(빈파일)
	char buf[DATASIZ] = "";
	sprintf( buf, "%s/%s", CONFIGDIR, syncname );
	fclose( fopen( buf, "w" ) );
	

    writeSyncList();
    return 1;
}


/* 함수 : 디버그를 위해서 syncList의 내용을 모두 출력하는 함수 
 */
void SyncManager::showList( void )
{
	setcolor( COLOR_SKY );
	printf("[=============================== SyncList ================================]\n");
	setcolor( COLOR_DARKRED );
	cout << "[SyncName]";
	setcolor( COLOR_DARKGREEN );
	cout << "[  Target  ]";
	setcolor( COLOR_BROWN );
	cout << "[  LocalPath  ]";
	setcolor( COLOR_PUPLE );
	cout << "[  RemotePath  ]"; 
	setcolor( COLOR_NONE );
	cout << "[Flag]";
	setcolor( COLOR_DARKSKY );
	cout << "[ LastSyncTime ]" << endl;
	for(int i = 0; i < (int)syncList.size(); i++)
	{
		setcolor( COLOR_DARKRED );
		printf("[%s]", syncList[i].syncName);
		setcolor( COLOR_DARKGREEN );
		printf("[%s]", syncList[i].target);
		setcolor( COLOR_BROWN );
		printf("[%s]", syncList[i].localPath);
		setcolor( COLOR_PUPLE );
		printf("[%s]", syncList[i].remotePath);
		//printf("[%s]", syncList[i].userId);
		//printf("[%s]", syncList[i].password);
		//printf("[%d]", syncList[i].fileSize);
		//printf("[%d]", syncList[i].option);
		setcolor( COLOR_NONE );
		printf("[%d]", syncList[i].Flag);
		setcolor( COLOR_DARKSKY );
		//printf("[%d]\n", (int)syncList[i].lastSyncTime);
		printf("[%s]\n", timeFormat(syncList[i].lastSyncTime) );
		setcolor( COLOR_NONE );
	}
}

/* 함수 : 디버그를 위해서 syncList의 내용을 모두 출력하는 함수 
 */
void SyncManager::debug( void )
{   
    printf("[================== SyncList =====================]\n");
	for(int i = 0; i < (int)syncList.size(); i++)
	{
		printf("* Number      : %d\n", i + 1);
		printf("* SyncName    : %s\n", syncList[i].syncName);
		printf("* Target      : %s\n", syncList[i].target);
		printf("* LocalPath   : %s\n", syncList[i].localPath);
		printf("* RemotePath  : %s\n", syncList[i].remotePath);
		printf("* UserId      : %s\n", syncList[i].userId);
		printf("* Password    : %s\n", syncList[i].password);
		printf("* Filesize    : %d\n", syncList[i].fileSize);
		printf("* Option      : %d\n", syncList[i].option);
		printf("* Flag        : %d\n", syncList[i].Flag);
		printf("* lastSyncTime: %d\n", (int)syncList[i].lastSyncTime);
		printf("==================================================\n");
	}
}

/* 함수 : syncList 특정 레코드 한개를 삭제하는 함수
 * 설명 : syncList를 한바퀴 돌면서 이름으로 검색하여 해당 이름을 갖는 레코드를 pop한다.
 * 리턴 : 0: 실패 1: 성공
 */
int SyncManager::delSyncNode( char *syncName )
{
	setFlag( syncName, FLAG_DELSYNC );
	return 1;
}

int SyncManager::delSync( int index )
{
	deque<syncListNode>		tmpList;
	syncListNode 			nodeTmp;

	tmpList = syncList;

	int i = 0;
	while( syncList.size() != 0 )
	{
		syncList.pop_back();
	}

	while( tmpList.size() != 0 )
	{
		if( i != index )
		{
			syncList.push_back( tmpList[0] );
		}
		tmpList.pop_front();

		i++;
	}


	return 1;
}
int SyncManager::delSyncNodeFromRemote( char *syncName )
{
	int indexOfSync;
	int sizeOfSyncList;

	char buf[DATASIZ] = "";
	sprintf( buf, "%s/%s", CONFIGDIR, syncName );

	indexOfSync = syncExist( syncName );			// syncName 을 갖는 레코드 검색

	if ( indexOfSync == 0 )							// syncName이 존재하지 않으면
	{
		puts("존재하지 않는 syncName 입니다.");
		return 1;
	}

	sizeOfSyncList = syncList.size();				// syncList Size 만큼 돈다

	for( int i = 0; i < sizeOfSyncList; i++)
	{
		slnTmp = syncList.front();					// syncList 처음부터 가져와서
		syncList.pop_front();

		if(indexOfSync != i + 1)					// syncName을 갖는 레코드를 제외하곤
			syncList.push_back( slnTmp );			// 다시 syncList deque에 넣는다.
		else
			cleanSyncListNode(slnTmp);				// 같은 syncName을 갖는 레코드의 메모리 해제
	}

	unlink( buf );

	writeSyncList();

	return 1;
}

/* 함수 : syncList의 syncName에 해당하는 레코드의 syncTime을 update 한다.
 * 인자 : update 시킬 레코드의 syncName, 바꿀 syncTime(동기화 시킨 Time)
 * 주의 : syncName이 없을 경우를 고려하지 않기 때문에 반드시 함수 호출전에
 * 		  syncName이 존재하는지 확인하자
 */
void SyncManager::updateSyncTime(char *syncName, time_t lastSyncTime)
{
	int indexOfSync;

	indexOfSync = syncExist( syncName );

	syncList[indexOfSync - 1].lastSyncTime = lastSyncTime;

	writeSyncList();
}

/* 함수 : syncList의 syncName에 해당하는 레코드의 FileSize를 update 하는 함수
 * 인자 : update 시킬 레코드의 syncName, 바꿀 fileSize(제한시킬 크기)
 *
 */
void SyncManager::updateFileSize(char *syncName, int fileSize)
{
	int indexOfSync;

	indexOfSync = syncExist( syncName ) - 1;

	syncList[indexOfSync].fileSize = fileSize;

	writeSyncList();
}

/* 함수 : syncList의 syncName에 해당하는 레코드의 option을 update 하는 함수
 * 인자 : update 시킬 레코드의 syncName, 바꿀 option
 */
void SyncManager::updateOption(char *syncName, int option)
{
	int indexOfSync;

	indexOfSync = syncExist( syncName ) - 1;

	syncList[indexOfSync].option = option;

	writeSyncList();
}

/* 함수 : SyncList의 Flag 필드를 다루는 함수
 * 인자 : update 시킬 레코드의 syncName 바꿀 플래그 옵션
 */
void SyncManager::setFlag(char *syncName, int flag)
{
	int indexOfSync;

	indexOfSync = syncExist( syncName ) - 1;

	// 아직 원격지에 리스트가 추가 되기 전이면 바로 삭제
	if( syncList[indexOfSync].Flag == FLAG_NEWSYNC )
	{
		delSync( indexOfSync );
	}
	else
	{
		syncList[indexOfSync].Flag = flag;
	}

/*
	switch(flag)
	{
	case FLAG_ON_ADD:
		syncList[indexOfSync].Flag |= FLAG_ADD;
		break;
	case FLAG_OFF_ADD:
		syncList[indexOfSync].Flag ^= FLAG_ADD;
		break;
	case FLAG_ON_DELETE:
		syncList[indexOfSync].Flag |= FLAG_DELETE;
		break;
	case FLAG_OFF_DELETE:
		syncList[indexOfSync].Flag ^= FLAG_DELETE;
		break;
	case FLAG_ON_CHANGE_OPTION:
		syncList[indexOfSync].Flag |= FLAG_CHANGE_OPTION;
		break;
	case FLAG_OFF_CHANGE_OPTION:
		syncList[indexOfSync].Flag ^= FLAG_CHANGE_OPTION;
		break;
	case FLAG_ON_CHANGE_FILESIZE:
		syncList[indexOfSync].Flag |= FLAG_CHANGE_FILESIZE;
		break;
	case FLAG_OFF_CHANGE_FILESIZE:
		syncList[indexOfSync].Flag ^= FLAG_CHANGE_FILESIZE;
	}
*/
	writeSyncList();
}

/* 함수 : syncName을 갖는 해당 syncList의 Flag가 set되어 있는지 확인하는 함수
 * 인자 : syncName, flag: 확인할 flag
 */
bool SyncManager::isFlag(char *syncName, int flag)
{
	int indexOfSync;

	indexOfSync = syncExist( syncName ) - 1;

	if((syncList[indexOfSync].Flag & flag) != 0)
		return true;

	return false;
}

/* 함수 : 사용자 및 네트워크를 통한 명령을 파싱 하는 함수
 * 설명 : 들어온 명령을 분리자로 자르는 일만 한다.
 * 인자 : 들어온 명령
 */
int SyncManager::cmdParsing(char *buf)
{
	char *tok;

	countOfCmd = 0;

	tok = strtok( buf, DELIM);

	if( !tok )
		return 0;

	while( tok )
	{
		cmd[ countOfCmd++ ] = strdup( tok );
		tok = strtok( NULL, DELIM );
	}

	return 1;
}

// 디버깅을 위한 명령 처리 함수
void *SyncManager::run( void *pthis )
{
	CommunicationManager 	*cm = (CommunicationManager *)pthis;
	SyncManager 			*sm = cm->sm;
	FileManager				*fm = cm->fm;

	sm->loadSyncList();

	while( 1 )
	{
		for( int i = 0; i < sm->syncList.size(); i++ )
		{
			if( !sm->lockState )
			{
				// SyncList 의 플래그가 Del 일 경우
				if ( sm->syncList[i].Flag == FLAG_DELSYNC )
				{
					// 타겟에 접속 후
					char cmd[DATASIZ] = "";
					sprintf( cmd, "target %s", sm->syncList[i].target );
					cm->getCmd( cmd );
					if( cm->getCmd( "connect" ) != CM_ACKOK )
					{
						continue;
					}

					// lock 실행 요청
					cm->getCmd( "lock" );
					if( cm->getCmd( "send lock" ) == CM_ACKOK )
					{
						bzero( cmd, sizeof(cmd) );
						sprintf( cmd, "send del %s %s", sm->syncList[i].syncName, "tmp" );

						if( cm->getCmd( cmd ) == CM_ACKOK )
						{
							// 로컬 싱크 삭제
							sm->delSyncNodeFromRemote( sm->syncList[i].syncName );
							sm->writeSyncList();
						}
						else
						{
							cout << "원격지의 리스트를 삭제 할 수 없습니다. " << endl;
						}

						cm->getCmd( "send unlock" );
					}

					// lock 해제 요청
					if( cm->isLock )
					{
						cm->getCmd( "unlock" );
					}

					cm->getCmd( "disconnect" );

				}
				// SyncList 의 플래그가 New 일 경우
				else if( sm->syncList[i].Flag == FLAG_NEWSYNC )
				{
					char cmd[DATASIZ] = "";
					sprintf( cmd, "target %s", sm->syncList[i].target );
					cm->getCmd( cmd );
					if( cm->getCmd( "connect" ) != CM_ACKOK )
					{
						continue;
					}

					// lock 실행 요청
					cm->getCmd( "lock" );
					if( cm->getCmd( "send lock" ) == CM_ACKOK )
					{
						bzero( cmd, sizeof(cmd) );
						sprintf( cmd, "send add %s %s %s %s %d",
								sm->syncList[i].syncName, 
								sm->syncList[i].target, 
								sm->syncList[i].remotePath, 
								sm->syncList[i].localPath,
								FLAG_SYNC );
						int result =  cm->getCmd( cmd );
						if( result == CM_ACKOK )
						{
							sm->syncList[i].Flag = FLAG_SYNC;
						}
						else
						{
							cout << "원격지에 리스트를 추가할 수 없습니다. ";
							cout << "리스트를 지웁니다" << endl;
							sm->delSyncNode( sm->syncList[i].syncName );
						}

						cm->getCmd( "send unlock" );

					}

					if( cm->isLock )
					{
						cm->getCmd( "unlock" );
					}

					cm->getCmd( "disconnect" );

					sm->writeSyncList();
					continue;
				}
				// SyncList 의 플래그가 Sync 일 경우
				else
				{
					sm->syncProc( cm, i );
				}
			}
			else
			{
				while( sm->lockState )
				{
					sleep(1);
				}
				i = 0;
				break;
			}
		}
		sleep(1);
	}
}

/*
 * 함수 : 동기화 프로세스
 * 설명 : 동기화 작업 중 파일 또는 폴더의 추가 삭제부분의 모듈이다.
 */
int SyncManager::syncProc( void *pthis, int index )
{
	CommunicationManager 	*cm = (CommunicationManager *)pthis;
	SyncManager 			*sm = cm->sm;
	FileManager				*fm = cm->fm;

	int i = index;

	bool updatetime = true;
	char cmd[DATASIZ] = "";

//	cout << "fm::isChanged " << sm->syncList[i].localPath << endl;
	//fm->isChanged( sm->syncList[i].localPath, &syncList[i].lastSyncTime );
	fm->isChanged( &sm->syncList[i] );

	if( fm->debugMode )
	{
		fm->showList(SHOW_CHANGE);
	}

	// 나의 타겟을 현재 SyncList의 타겟으로 세팅
	sprintf( cmd, "target %s", syncList[i].target );
	cm->getCmd( cmd );

	// 바뀐 파일이 있을 경우에만 동기화 작업을 수행
	// 바뀐 파일이 없을 경우 다음 SyncList로 이동
	if( fm->changeList.empty() )
	{
		return 0;
	}

	if( cm->getCmd( "connect" ) != CM_ACKOK )
	{
		return 0;
	}

	// lock 실행 요청
	cm->getCmd( "lock" );
	if( cm->getCmd( "send lock" ) == CM_ACKOK )
	{
		for( int j = 0; j < fm->changeList.size(); j++ )
		{
			bzero( cmd, sizeof(cmd) );
			int result;

			switch( fm->changeList[j].type )
			{
				case CI_DIRADD :
					sprintf( cmd, "adddir %s", fm->changeList[j].rPath );
/*		이름 변경
					sprintf( cmd, "adddir %s", gen_rpath( syncList[i].localPath,
														syncList[i].remotepath,
														fm->changeList[j].name ) );
*/
					result = cm->getCmd( cmd );
					break;
				case CI_FILEADD :
					char md5tmp[DATASIZ];
					bzero( md5tmp, sizeof(md5tmp) );
//					strcpy( md5tmp, md5file( fm->changeList[j].name ) );
//					sprintf( cmd, "addfile %s %s", fm->changeList[j].name, md5tmp );
					strcpy( md5tmp, md5file( fm->changeList[j].lPath ) );
					sprintf( cmd, "addfile %s %s", fm->changeList[j].lPath, md5tmp );
					result = cm->getCmd( cmd );

					if( result == CM_ACKOK )
					{
						// CM을 이용한 파일 전송
						cm->sendFile( fm->changeList[j].lPath, fm->changeList[j].rPath, md5tmp );
/*		이름 변경		cm->sendFile( fm->changeList[j].name,
						gen_rpath( syncList[i].localPath,
						syncList[i].remotePath,
						fm->changeList[j].name ), md5tmp ); 
*/									
					}
					else if( result == CM_MD5HASH )
					{
					}
					else
					{
						updatetime 	= false;
					}

					break;
				case CI_FILEDEL :
					sprintf( cmd, "delfile %s", fm->changeList[j].rPath );
					result = cm->getCmd( cmd );
					break;
				case CI_DIRDEL :
					sprintf( cmd, "deldir %s", fm->changeList[j].rPath );
					result = cm->getCmd( cmd );
					break;
			}

			if( result == CM_ACKNO )
			{
				updatetime = false;
			}
			if( !updatetime )
			{
				break;
			}
		}

		if( updatetime )
		{
			sprintf( cmd, "send update %s", syncList[i].syncName );
			if( cm->getCmd( cmd ) )
				updateSyncTime( syncList[i].syncName, time(NULL) );
			//updateSyncTime( syncList[i].syncName, cm->lockTime );
			fm->writeCFL( syncList[i].syncName );
		}
		cm->getCmd( "send unlock" );
	}

	// lock 해제 요청
	if( cm->isLock )
	{
		cm->getCmd( "unlock" );
	}
	cm->getCmd( "disconnect" );
}

/* 함수 : 자른 명령을 실제로 처리하는 함수
 * 
 */
int SyncManager::executeCommand( char **cmd )
{
	// sync list에 레코드를 추가하는 명령
	// add [SYNCNAME] [TARGET] [LOCALPATH] [REMOTEPATH] [USERID] [PASSWORD]
	// [FILESIZE] [OPTION]
	if ( strcmp ( cmd[0], "add") == 0 )
	{
		int n_option, n_filesize;

		n_filesize = atoi( cmd[7] );
		n_option = atoi( cmd[8] );

		addSync( cmd[1], cmd[2], cmd[3], cmd[4], cmd[5], cmd[6], n_filesize, n_option );
	}
	// sync list의 특정 레코드를 지우는 명령
	// delete [SYNCNAME]
	else if ( strcmp ( cmd[0], "delete_r") == 0 )
	{
		delSyncNode(cmd[1]);	// 해당이름의 레코드를 삭제한다.
	}
	// option과 filesize를 바꾸는 명령
	// change -option [SYNCNAME] [OPTION]
	// change -filesize [SYNCNAME] [FILESIZE]
	else if ( strcmp ( cmd[0], "change_r") == 0 )
	{
		if ( strcmp ( cmd[1], "-option") == 0 )
		{
			int n_option;

			n_option = atoi( cmd[3] );
			
			updateOption( cmd[2], n_option ) ;	// 해당이름의 레코드의 option을 update 시킨다.
		}

	}
	// sync list를 보는 함수
	// view synclist
	else if ( strcmp ( cmd[0], "view") == 0 )
	{
		if ( strcmp (cmd[1], "synclist" ) == 0 )
		{
			debug();	// 출력한다.
		}
	}

	// Flag를 2로 세팅 한다.
	else if ( strcmp ( cmd[0], "delete" ) == 0 )
	{
		;	
	}

	else if ( strcmp ( cmd[0], "change" ) == 0 )
	{
		;
	}

	else if ( strcmp ( cmd[0], "exit" ) == 0 )
	{
		return 0;
	}
	else
	{
		cout << "없는 명령어 입니다." << endl; 
	}

	return 1;
}

/* 함수 : 명령어 인자를 저장하고 있는 cmd파일의 메모리를 해제하는 함수
 *
 */
void SyncManager::clearCmd( char **cmd )
{
	for(int i = 0; i < countOfCmd; i++)
	{
		if( cmd[i] != NULL )
			delete cmd[i];

		cmd[i] = NULL;
	}

	countOfCmd = 0;
}

// 원격지 sm에게 syncList의 내용을 추가하라고하는 add하는 명령
// 원격지 sm에게 syncList의 특정 syncName을 가지고 있는 레코드를 지우라는
// 명령
